@@ -12,7 +12,7 @@ use Carp;
 use strict;
 use warnings;
 
-our @EXPORT_OK = qw(debug $DEBUG);
+our @EXPORT_OK = qw(debug $DEBUG carp_once);
 our %EXPORT_TAGS = (all => [ @EXPORT_OK ]);
 
 our ($DEBUG, $DEBUG_fh);
@@ -20,9 +20,9 @@ our $VERSION = $Forks::Super::Util::VERSION;
 
 open($DEBUG_fh, '>&STDERR')
   or $DEBUG_fh = *STDERR
-  or carp "Forks::Super: Debugging not available in module!\n";
+  or carp_once("Forks::Super: Debugging not available in module!\n");
 $DEBUG_fh->autoflush(1);
-$DEBUG = $ENV{FORKS_SUPER_DEBUG} || "0";
+$DEBUG = $ENV{FORKS_SUPER_DEBUG} || '0';
 
 sub init {
 }
@@ -33,4 +33,18 @@ sub debug {
   return;
 }
 
+# sometimes we only want to print a warning message once
+our %_CARPED = ();
+sub carp_once {
+  my @msg = @_;
+  my ($p,$f,$l) = caller;
+  my $z = '';
+  if (ref $msg[0] eq 'ARRAY') {
+    $z = join ';', @{$msg[0]};
+    shift;
+  }
+  return if $_CARPED{"$p:$f:$l:$z"}++;
+  carp @msg;
+}
+
 1;
